# -*- coding: utf-8 -*-
# @Project: 芒果测试平台
# @Description: 
# @Time   : 2024-05-23 15:04
# @Author : 毛鹏
# -*- coding: utf-8 -*-
# @Project: 芒果测试平台
# @Description:
# @Time   : 2024-04-24 10:43
# @Author : 毛鹏
import ctypes
import os
import string
import threading
import traceback
from typing import Optional

import time
from playwright._impl._errors import Error
from playwright.sync_api import sync_playwright, Page, BrowserContext, Browser, Playwright, Request, Route

from ....enums import BrowserTypeEnum
from ....exceptions import MangoAutomationError
from ....exceptions._error_msg import ERROR_MSG_0057, ERROR_MSG_0008, ERROR_MSG_0062, ERROR_MSG_0009, ERROR_MSG_0055



class SyncWebNewBrowser:

    def __init__(self,
                 web_type: int,
                 web_path: str | None = None,
                 web_max=False,
                 web_headers=False,
                 web_recording=False,
                 web_h5=None,
                 is_header_intercept=False,
                 web_is_default=False,
                 videos_path=None
                 ):
        self.lock = threading.Lock()
        self.web_type = web_type
        self.web_path = web_path
        self.web_max = web_max
        self.web_headers = web_headers
        self.web_recording = web_recording
        self.web_h5 = web_h5
        self.web_is_default = web_is_default
        self.videos_path = videos_path
        self.is_header_intercept = is_header_intercept
        self.browser_path = ['chrome.exe', 'msedge.exe', 'firefox.exe', '苹果', '360se.exe']
        self.browser: Optional[None | Browser] = None
        self.playwright: Optional[None | Playwright] = None

    def new_web_page(self, count=0) -> tuple[BrowserContext, Page]:
        if self.browser is None:
            with self.lock:
                self.browser = self.new_browser()
                time.sleep(1)
        try:
            context = self.new_context()
            page = self.new_page(context)
            return context, page
        except Exception as error:
            self.browser = None
            traceback.format_exc()
            if count >= 3:
                raise MangoAutomationError(*ERROR_MSG_0057)
            else:
                return self.new_web_page(count=count + 1)

    def new_browser(self) -> Browser:
        self.playwright = sync_playwright().start()
        if self.web_type == BrowserTypeEnum.CHROMIUM.value or self.web_type == BrowserTypeEnum.EDGE.value:
            browser = self.playwright.chromium
        elif self.web_type == BrowserTypeEnum.FIREFOX.value:
            browser = self.playwright.firefox
        elif self.web_type == BrowserTypeEnum.WEBKIT.value:
            browser = self.playwright.webkit
        else:
            raise MangoAutomationError(*ERROR_MSG_0008)
        if self.web_is_default:
            try:
                return browser.launch()
            except Error:
                raise MangoAutomationError(*ERROR_MSG_0062)
        else:
            try:
                if self.web_max:
                    return browser.launch(
                        headless=self.web_headers,
                        executable_path=self.web_path if self.web_path else self.__search_path(),
                        args=['--start-maximized']
                    )
                else:
                    return browser.launch(
                        headless=self.web_headers,
                        executable_path=self.web_path if self.web_path else self.__search_path()
                    )
            except Error:
                raise MangoAutomationError(*ERROR_MSG_0009, value=(self.web_path,))

    def new_context(self) -> BrowserContext:
        args_dict = {
            'no_viewport': True,
        }

        if self.web_recording and self.videos_path:
            args_dict['record_video_dir'] = self.videos_path

        if self.web_h5:
            del args_dict['no_viewport']
            args_dict.update(self.playwright.devices[self.web_h5])
        return self.browser.new_context(**args_dict)

    def new_page(self, context: BrowserContext) -> Page:
        try:
            page = context.new_page()
            if self.is_header_intercept:
                page.route("**/*", self.wen_intercept_request)  # 应用拦截函数到页面的所有请求
            return page
        except Error:
            raise MangoAutomationError(*ERROR_MSG_0009, value=(self.web_path,))

    def close(self):
        if self.browser:
            self.browser.close()

    def __search_path(self, ):
        drives = []
        for letter in string.ascii_uppercase:
            drive = f"{letter}:\\"
            if ctypes.windll.kernel32.GetDriveTypeW(drive) == 3:
                drives.append(drive)
        for i in drives:
            for root, dirs, files in os.walk(i):
                if self.browser_path[self.web_type] in files:
                    return os.path.join(root, self.browser_path[self.web_type])

        raise MangoAutomationError(*ERROR_MSG_0055)

    def wen_intercept_request(self, route: Route, request: Request):
        pass

    def wen_recording_api(self, request: Request, project_product: int):
        pass



# -*- coding: utf-8 -*-
# @Project: 芒果测试平台
# @Description: 
# @Time   : 2025-04-12 15:55
# @Author : 毛鹏
import os
import traceback

import time
from playwright._impl._errors import TargetClosedError, Error, TimeoutError

from mangotools.assertion import PublicAssertion
from mangotools.decorator import sync_retry
from mangotools.enums import StatusEnum
from ..enums import ElementOperationEnum, DriveTypeEnum
from ..exceptions import MangoAutomationError
from ..exceptions._error_msg import *
from ..models import ElementResultModel, ElementModel
from ..uidrive.android import AndroidDriver
from ..uidrive.web.sync_web import SyncWebDevice, SyncWebAssertion


class SyncElement(SyncWebDevice, AndroidDriver):

    def __init__(self, base_data, element_model: ElementModel, drive_type: int, element_data: dict | None = None):
        super().__init__(base_data)
        self.element_data = element_data
        self.element_model = element_model
        self.drive_type = drive_type
        self.ope_name = element_model.name if element_model.name else element_model.ope_key
        self.element_result_model = ElementResultModel(
            id=self.element_model.id,
            name=self.element_model.name,
            loc=self.element_model.loc,
            exp=self.element_model.exp,
            sub=self.element_model.sub,
            sleep=self.element_model.sleep,

            type=self.element_model.type,
            ope_key=self.element_model.ope_key,
            sql=self.element_model.sql,
            key_list=self.element_model.key_list,
            key=self.element_model.key,
            value=self.element_model.value,

            status=StatusEnum.FAIL.value,
        )

    def open_device(self, is_open: bool = False):
        if self.drive_type == DriveTypeEnum.WEB.value:
            self.open_url(is_open)
        elif self.drive_type == DriveTypeEnum.ANDROID.value:
            self.open_app()
        elif self.drive_type == DriveTypeEnum.DESKTOP.value:
            pass
        else:
            self.base_data.log.error(f'不存在这个类型，如果是非管理员看到这种提示，请联系管理员')
            raise Exception('不存在的设备类型')

    def clean_data(self):
        try:
            for field_name, field_value in self.element_model:
                if field_value is None:
                    continue
                if field_name == "ope_value":
                    for method_model in field_value:
                        for method_field, method_val in method_model:
                            if isinstance(method_val, str):
                                setattr(method_model, method_field, self.base_data.test_data.replace(method_val))
                elif isinstance(field_value, str):
                    setattr(self.element_model, field_name, self.base_data.test_data.replace(field_value))
        except MangoAutomationError as error:
            self.base_data.log.debug(f'操作元素解析数据失败，类型：{type(error)}, 详情：{error}')
            raise MangoAutomationError(error.code, error.msg)

    def element_main(self) -> ElementResultModel:
        try:
            self.__main()
            if self.element_model.sleep:
                time.sleep(self.element_model.sleep)
            self.element_result_model.status = StatusEnum.SUCCESS.value
            self.element_result_model.error_message = None
        except TargetClosedError as error:
            self.base_data.setup()
            self.base_data.log.debug(
                f'浏览器关闭异常，类型：{type(error)}，失败详情：{error}，失败明细：{traceback.format_exc()}')
            self.element_result_model.status = StatusEnum.FAIL.value
            self.element_result_model.error_message = '浏览器被关闭，请不要认关闭浏览器，非认为管理请联系管理员解决！'
        except MangoAutomationError as error:
            self.__error(error.msg)
            self.base_data.log.debug(f'已知异常，类型：{type(error)}，失败详情：{error}')
        except Error as error:
            self.__error(f'未知错误失败，请检查测试数据，如果需要明确的提示请联系管理员，提示：{error.message}')
            self.base_data.log.error(
                f'未知错误捕获-1，类型：{type(error)}，失败详情：{error}，失败明细：{traceback.format_exc()}')
        except Exception as error:
            error_msg = f'未知错误失败，请检查测试数据，如果需要明确的提示请联系管理员，提示：{error.args}'
            if hasattr(error, 'msg'):
                error_msg = error.msg
            self.__error(error_msg)
            self.base_data.log.error(
                f'未知错误捕获-2，类型：{type(error)}，失败详情：{error}，失败明细：{traceback.format_exc()}')
        return self.element_result_model

    @sync_retry()
    def __main(self):
        self.clean_data()
        if self.element_model.type == ElementOperationEnum.OPE.value:
            self.__ope()
        elif self.element_model.type == ElementOperationEnum.ASS.value:
            self.__ass()
        elif self.element_model.type == ElementOperationEnum.SQL.value:
            self.__sql()
        elif self.element_model.type == ElementOperationEnum.CUSTOM.value:
            self.__custom()
        else:
            raise MangoAutomationError(*ERROR_MSG_0015)

    def __ope(self):
        method_name = getattr(self.element_model, 'ope_key', None)
        if not method_name:
            self.base_data.log.debug('操作失败-1，ope_key 不存在或为空')
            raise MangoAutomationError(*ERROR_MSG_0048)
        if not hasattr(self, method_name):
            self.base_data.log.debug(f'操作失败-2，方法不存在: {method_name}')
            raise MangoAutomationError(*ERROR_MSG_0048)
        if not callable(getattr(self, method_name)):
            self.base_data.log.debug(f'操作失败-3，属性不可调用: {method_name}')
            raise MangoAutomationError(*ERROR_MSG_0048)
        if self.element_model.ope_value is None:
            raise MangoAutomationError(*ERROR_MSG_0054)

        self.__ope_value()
        if self.drive_type == DriveTypeEnum.WEB.value:
            self.web_action_element(
                self.element_model.name,
                self.element_model.ope_key,
                {i.f: i.v for i in self.element_model.ope_value}
            )
        elif self.drive_type == DriveTypeEnum.ANDROID.value:
            self.a_action_element(
                self.element_model.name,
                self.element_model.ope_key,
                {i.f: i.v for i in self.element_model.ope_value}
            )
        else:
            pass
        for i in self.element_model.ope_value:
            if i.d:
                self.element_result_model.ope_value[i.f] = i.v

    def __ass(self):
        if self.element_model.ope_value is None:
            raise MangoAutomationError(*ERROR_MSG_0053)
        self.__ope_value(True)
        if self.drive_type == DriveTypeEnum.WEB.value:
            self.web_assertion_element(
                self.element_model.name,
                self.element_model.ope_key,
                {i.f: i.v for i in self.element_model.ope_value}
            )
        elif self.drive_type == DriveTypeEnum.ANDROID.value:
            self.a_assertion_element(
                self.element_model.name,
                self.element_model.ope_key,
                {i.f: i.v for i in self.element_model.ope_value}
            )
        else:
            pass
        for i in self.element_model.ope_value:
            if i.d:
                self.element_result_model.ope_value[i.f] = i.v

    def __sql(self):
        if not self.element_data:
            sql = self.base_data.test_data.replace(self.element_model.sql)
            key_list = self.element_model.key_list
        else:
            sql = self.base_data.test_data.replace(self.element_data.get('sql'))
            key_list = self.element_data.get('key_list')
        if self.base_data.mysql_connect:
            result_list: list[dict] = self.base_data.mysql_connect.condition_execute(sql)
            if isinstance(result_list, list):
                for result in result_list:
                    try:
                        for value, key in zip(result, key_list):
                            self.base_data.test_data.set_cache(key, result.get(value))
                    except SyntaxError as error:
                        self.base_data.log.debug(
                            f'SQL执行失败-1，类型：{type(error)}，失败详情：{error}，失败明细：{traceback.format_exc()}')
                        raise MangoAutomationError(*ERROR_MSG_0038)

                if not result_list:
                    raise MangoAutomationError(*ERROR_MSG_0036, value=(self.element_model.sql,))

    def __custom(self):
        if not self.element_data:
            key = self.element_model.key
            value = self.element_model.value
        else:
            key = self.element_data.get('key')
            value = self.element_data.get('value')
        self.base_data.test_data.set_cache(key, self.base_data.test_data.replace(value))

    def __ope_value(self, is_ass: bool = False):
        try:
            ope_key = 'actual' if is_ass else 'locating'
            for i in self.element_model.ope_value:
                if i.f == ope_key and self.element_model.loc:
                    find_params = {
                        'name': self.element_model.name,
                        '_type': self.element_model.type,
                        'exp': self.element_model.exp,
                        'loc': self.element_model.loc,
                        'sub': self.element_model.sub
                    }
                    if self.drive_type == DriveTypeEnum.WEB.value:
                        loc, \
                            self.element_result_model.ele_quantity, \
                            self.element_result_model.element_text \
                            = self.web_find_ele(**find_params, is_iframe=self.element_model.is_iframe)
                    elif self.drive_type == DriveTypeEnum.ANDROID.value:
                        loc, \
                            self.element_result_model.ele_quantity, \
                            self.element_result_model.element_text \
                            = self.a_find_ele(**find_params)
                    else:
                        loc, \
                            self.element_result_model.ele_quantity, \
                            self.element_result_model.element_text \
                            = None, 0, None

                    if is_ass:
                        if callable(getattr(SyncWebAssertion, self.element_model.ope_key, None)):
                            i.v = loc
                        elif callable(getattr(PublicAssertion, self.element_model.ope_key, None)):
                            i.v = self.element_result_model.element_text
                    else:
                        i.v = loc
                else:
                    if self.element_data:
                        for ele_name, case_data in self.element_data.items():
                            if ele_name == i.f:
                                value = case_data
                                i.v = self.base_data.test_data.replace(value)

        except AttributeError as error:
            self.base_data.log.debug(
                f'获取操作值失败-1，类型：{type(error)}，失败详情：{error}，失败明细：{traceback.format_exc()}')
            raise MangoAutomationError(*ERROR_MSG_0027)

    def __error(self, msg: str):
        self.element_result_model.status = StatusEnum.FAIL.value
        self.element_result_model.error_message = msg
        self.base_data.log.debug(
            f"""
            元素操作失败----->
            元 素 对 象：{self.element_model.model_dump() if self.element_model else self.element_model}
            元素测试结果：{
            self.element_result_model.model_dump() if self.element_result_model else self.element_result_model}
            """
        )
        file_name = f'失败截图-{self.element_model.name}{self.base_data.test_data.get_time_for_min()}.jpg'
        file_path = os.path.join(self.base_data.screenshot_path, file_name)
        self.element_result_model.picture_path = file_path
        self.element_result_model.picture_name = file_name
        self.__error_screenshot(file_path)

    def __error_screenshot(self, file_path):
        if self.drive_type == DriveTypeEnum.WEB.value:
            try:
                self.w_screenshot(file_path)
            except (TargetClosedError, TimeoutError) as error:
                self.base_data.log.debug(
                    f'截图出现异常失败-1，类型：{type(error)}，失败详情：{error}，失败明细：{traceback.format_exc()}')
                self.base_data.setup()
                raise MangoAutomationError(*ERROR_MSG_0010)
            except AttributeError as error:
                self.base_data.log.debug(
                    f'截图出现异常失败-2，类型：{type(error)}，失败详情：{error}，失败明细：{traceback.format_exc()}')
                self.base_data.setup()
                raise MangoAutomationError(*ERROR_MSG_0010)
        elif self.drive_type == DriveTypeEnum.ANDROID.value:
            self.a_screenshot(file_path)
        else:
            pass



# -*- coding: utf-8 -*-
# @Project: 芒果测试平台# @Description:
# @Time   : 2023-09-09 23:17
# @Author : 毛鹏
import os.path

import time
from uiautomator2 import UiObject
from uiautomator2.xpath import XPathSelector

from ...exceptions import MangoAutomationError
from ...exceptions._error_msg import ERROR_MSG_0043, ERROR_MSG_0044
from ...tools import Meta
from ...uidrive._base_data import BaseData
from mangotools.decorator import sync_method_callback
from mangotools.models import MethodModel


class AndroidElement(metaclass=Meta):
    """元素操作"""

    def __init__(self, base_data: BaseData):
        self.base_data = base_data

    @sync_method_callback('android', '元素操作', 0, [
        MethodModel(f='locating')])
    def a_click(self, locating: UiObject | XPathSelector):
        """元素单击"""
        locating.click()

    @sync_method_callback('android', '元素操作', 1, [
        MethodModel(f='locating')])
    def a_double_click(self, locating: UiObject):
        """元素双击"""
        locating.click()

    @sync_method_callback('android', '元素操作', 2, [
        MethodModel(f='locating'), MethodModel(f='text', p='请输入内容', d=True)])
    def a_input(self, locating: UiObject, text):
        """单击输入"""
        locating.click()
        self.base_data.android.set_fastinput_ime(True)
        time.sleep(1)
        self.base_data.android.send_keys(text)

    @sync_method_callback('android', '元素操作', 3, [
        MethodModel(f='locating'), MethodModel(f='text', p='请输入内容', d=True)])
    def a_set_text(self, locating: UiObject, text):
        """设置文本"""
        locating.set_text(text)

    @sync_method_callback('android', '元素操作', 4, [
        MethodModel(f='locating'),
        MethodModel(f='x', p='请输入x坐标', d=True),
        MethodModel(f='y', p='请输入y坐标', d=True)])
    def a_click_coord(self, x, y):
        """坐标单击"""
        self.base_data.android.click(x, y)

    @sync_method_callback('android', '元素操作', 5, [
        MethodModel(f='x', p='请输入x坐标', d=True),
        MethodModel(f='y', p='请输入y坐标', d=True)])
    def a_double_click_coord(self, x, y):
        """坐标双击"""
        self.base_data.android.double_click(x, y)

    @sync_method_callback('android', '元素操作', 6, [
        MethodModel(f='locating'),
        MethodModel(f='time_', p='请输入长按坐标时间', d=True)])
    def a_long_click(self, locating: UiObject, time_):
        """长按元素"""
        locating.long_click(duration=float(time_))

    @sync_method_callback('android', '元素操作', 7, [MethodModel(f='locating')])
    def a_clear_text(self, locating: UiObject):
        """清空输入框"""
        locating.clear_text()

    @sync_method_callback('android', '元素操作', 8, [
        MethodModel(f='locating'), MethodModel(f='set_cache_key', p='请输入元素文本存储的key', d=True)])
    def a_get_text(self, locating: UiObject, set_cache_key=None):
        """获取元素文本"""
        value = locating.get_text()
        if set_cache_key:
            self.base_data.test_data.set_cache(key=set_cache_key, value=value)
        return value

    @sync_method_callback('android', '元素操作', 9, [
        MethodModel(f='locating'),
        MethodModel(f='file_name', p='请输入元素截图存储的名称，后续可以通过名称获取', d=True)])
    def a_element_screenshot(self, locating: UiObject, file_name: str):
        """元素截图"""
        im = locating.screenshot()
        file_path = os.path.join(self.base_data.screenshot_path, file_name)
        self.base_data.test_data.set_cache(file_name, file_path)
        im.save(file_path)

    @sync_method_callback('android', '元素操作', 10, [
        MethodModel(f='locating')])
    def a_pinch_in(self, locating: UiObject):
        """元素缩小"""
        locating.pinch_in()

    @sync_method_callback('android', '元素操作', 11, [
        MethodModel(f='locating')])
    def a_pinch_out(self, locating: UiObject):
        """元素放大"""
        locating.pinch_out()

    @sync_method_callback('android', '元素操作', 12, [
        MethodModel(f='locating'), MethodModel(f='time_', p='请输入等待元素出现的时间', d=True)])
    def a_wait(self, locating: UiObject, time_):
        """等待元素出现"""
        if not locating.wait(timeout=float(time_)):
            raise MangoAutomationError(*ERROR_MSG_0043)

    @sync_method_callback('android', '元素操作', 13, [
        MethodModel(f='locating'), MethodModel(f='time_', p='请输入等待元素消失的时间', d=True)])
    def a_wait_gone(self, locating: UiObject, time_: str):
        """等待元素消失"""
        if not locating.wait_gone(timeout=float(time_)):
            raise MangoAutomationError(*ERROR_MSG_0044)

    @sync_method_callback('android', '元素操作', 14, [
        MethodModel(f='locating'), MethodModel(f='locating2')])
    def a_drag_to_ele(self, locating: UiObject, locating2: UiObject):
        """拖动A元素到达B元素上"""
        locating.drag_to(locating2)

    @sync_method_callback('android', '元素操作', 15, [
        MethodModel(f='locating'),
        MethodModel(f='x', p='请输入x坐标', d=True),
        MethodModel(f='y', p='请输入y坐标', d=True)])
    def a_drag_to_coord(self, locating: UiObject, x, y):
        """拖动元素到坐标上"""
        locating.drag_to(x, y)

    @sync_method_callback('android', '元素操作', 16, [MethodModel(f='locating')])
    def a_swipe_right(self, locating: UiObject):
        """元素内向右滑动"""
        locating.swipe('right')

    @sync_method_callback('android', '元素操作', 17, [MethodModel(f='locating')])
    def a_swipe_left(self, locating: UiObject):
        """元素内向左滑动"""
        locating.swipe('left')

    @sync_method_callback('android', '元素操作', 18, [MethodModel(f='locating')])
    def a_swipe_up(self, locating: UiObject):
        """元素内向上滑动"""
        locating.swipe('up')

    @sync_method_callback('android', '元素操作', 19, [MethodModel(f='locating')])
    def a_swipe_ele(self, locating: UiObject):
        """元素内向下滑动"""
        locating.swipe('down')

    @sync_method_callback('android', '元素操作', 20, [
        MethodModel(f='locating'),
        MethodModel(f='x_key', p='请输入x坐标', d=True),
        MethodModel(f='y_key', p='请输入y坐标', d=True)])
    def a_get_center(self, locating: UiObject, x_key, y_key):
        """提取元素坐标"""
        x, y = locating.center()
        if x_key and y_key:
            self.base_data.test_data.set_cache(key=x_key, value=x)
            self.base_data.test_data.set_cache(key=y_key, value=y)
        return x, y



# -*- coding: utf-8 -*-
# @Project: 芒果测试平台# @Description:
# @Time   : 2023-09-09 23:17
# @Author : 毛鹏
import sys

from ..models._ui_model import ElementModel, ElementResultModel

python_version = sys.version_info
if f"{python_version.major}.{python_version.minor}" != "3.10":
    raise Exception("必须使用>Python3.10.4")
__all__ = [
    'ElementModel',
    'ElementResultModel',
]



# -*- coding: utf-8 -*-
# @Project: 芒果测试平台
# @Description: # @Time   : 2023-07-15 12:02
# @Author : 毛鹏
from typing import Optional

import sys

if not sys.platform.startswith('win32'):
    print("警告: uiautomation 仅支持 Windows，当前环境已自动跳过")


    class Auto:
        pass


    auto = Auto()


    class Control:
        pass
else:

    from uiautomation import Control
    import uiautomation

from mangoautomation.uidrive._base_data import BaseData
from mangoautomation.uidrive.pc.element import WinElement
from mangoautomation.uidrive.pc.input_device import WinDeviceInput


class WinDriver(WinElement, WinDeviceInput):
    def __init__(self, base_data: BaseData):
        super().__init__(base_data)

    def find_element(
            self,
            *,
            control_type: str = "Control",
            name: Optional[str] = None,
            automation_id: Optional[str] = None,
            depth: int = 2,
            parent: Optional[Control] = None,
            **extra_attrs
    ) -> Control:

        if not any([name, automation_id, extra_attrs]):
            raise ValueError("必须提供至少一个定位参数(name/automation_id/其他属性)")

        # 设置默认父控件
        parent = parent or self.base_data.window.GetParentControl()

        # 构造搜索参数
        search_params = {
            "searchDepth": depth,
            "Timeout": 10 * 1000,
            **{
                key.capitalize() if key != "className" else "ClassName": value
                for key, value in {
                    "name": name,
                    "automationId": automation_id,
                    **extra_attrs
                }.items()
                if value is not None
            }
        }

        # 获取控件类并查找
        control_class = getattr(uiautomation, f"{control_type}Control", Control)
        control = control_class(parent=parent, **search_params)

        if not control.Exists():
            raise ElementNotFoundError(
                f"未找到控件: type={control_type}, params={search_params}"
            )
        return control


class ElementNotFoundError(Exception):
    pass



# -*- coding: utf-8 -*-
# @Project: 芒果测试平台# @Description:
# @Time   : 2023-09-09 23:17
# @Author : 毛鹏
import time

from mangotools.decorator import sync_method_callback
from mangotools.models import MethodModel
from ...exceptions import MangoAutomationError
from ...exceptions._error_msg import ERROR_MSG_0046
from ...tools import Meta
from ...uidrive._base_data import BaseData


class AndroidApplication(metaclass=Meta):
    """应用操作"""

    def __init__(self, base_data: BaseData):
        self.base_data = base_data

    def is_app_installed(self, package_name: str) -> bool:
        return any(package_name in str(i) for i in self.base_data.android.shell("pm list packages"))

    @sync_method_callback('android', '应用操作', [
        MethodModel(f='package_name', p='请输入应用名称', d=True)])
    def a_start_app(self, package_name: str):
        """启动应用"""
        if not package_name:
            raise MangoAutomationError(*ERROR_MSG_0046)
        if not self.is_app_installed(package_name):
            raise MangoAutomationError(*ERROR_MSG_0046)
        self.base_data.android.app_start(package_name)
        time.sleep(4)

    @sync_method_callback('android', '应用操作', [
        MethodModel(f='package_name', p='请输入应用名称', d=True)])
    def a_close_app(self, package_name: str):
        """关闭应用"""
        if not package_name:
            raise MangoAutomationError(*ERROR_MSG_0046)
        if not self.is_app_installed(package_name):
            raise MangoAutomationError(*ERROR_MSG_0046)

        self.base_data.android.app_stop(package_name)

    @sync_method_callback('android', '应用操作', [
        MethodModel(f='package_name', p='请输入应用名称', d=True)])
    def a_clear_app(self, package_name: str):
        """清除app数据"""
        if not package_name:
            raise MangoAutomationError(*ERROR_MSG_0046)
        if not self.is_app_installed(package_name):
            raise MangoAutomationError(*ERROR_MSG_0046)

        self.base_data.android.app_clear(package_name)

    @sync_method_callback('android', '应用操作', )
    def a_app_stop_all(self):
        """停止所有app"""
        self.base_data.android.app_stop_all()

    @sync_method_callback('android', '应用操作', [
        MethodModel(f='package_name', p='请输入应用名称列表', d=True)])
    def a_app_stop_appoint(self, package_name_list: list):
        """停止除指定app外所有app"""
        for i in package_name_list:
            if not self.is_app_installed(i):
                raise MangoAutomationError(*ERROR_MSG_0046)
        self.base_data.android.app_stop_all(excludes=package_name_list)



# -*- coding: utf-8 -*-
# @Project: 芒果测试平台
# @Description: 错误消息统一管理
# @Time   : 2024-02-01 10:00
# @Author : 毛鹏

ERROR_MSG_0001 = (301, '连接错误，请检查IP，端口，账号，密码')
ERROR_MSG_0002 = (302, '缓存中无法获取到：{}的值')
ERROR_MSG_0003 = (303, 'jsonpath表达式错误，请检查表达式：{}')
ERROR_MSG_0004 = (304, '数据不可以被转换成json')
ERROR_MSG_0005 = (318, '断言失败，断言输入值是：空字符串、null或不满足函数接受类型')
ERROR_MSG_0006 = (306, '随机字符串函数只支持传入int类型数字')
ERROR_MSG_0007 = (307, '请先启动容器')
ERROR_MSG_0008 = (308, '浏览器类型不正确，请联系管理员检查')
ERROR_MSG_0009 = (309, '浏览器路径不正确，请手动设置一个正确的浏览器路径，现在的路径：{}')
ERROR_MSG_0010 = (310, '浏览器被关闭，请重试')
ERROR_MSG_0011 = (311, '操作元素:{} 超时，请检查元素可见但是是否可操作，或元素在iframe中，或调整定位方式')
ERROR_MSG_0012 = (312, '操作失败，请检查输入的值是否有空字符串，null或不满足传参的值')
ERROR_MSG_0013 = (
    313, '打开url时超时，分为2个问题：1.启用的浏览器过多，应当减少浏览器并发个数；2.自行查看为啥60秒还未打开：{}')
ERROR_MSG_0014 = (314, '企微发送失败，请检查企微发送配置')
ERROR_MSG_0015 = (315, '的类型错误，请联系管理员进行排查问题')
ERROR_MSG_0016 = (316, '邮件发送失败，请检查邮件的配置')
ERROR_MSG_0017 = (317, '断言失败，提示： {}')
ERROR_MSG_0018 = (318, 'webhook的链接错误！')
ERROR_MSG_0019 = (319, '配置了sql断言，但是并没有配置mysql的数据库配置，请检查！')
ERROR_MSG_0020 = (320, '没有更多的断言方式，请检查是否选择了非该设备类型的元素定位方式')
ERROR_MSG_0021 = (321, '元素未找到，请检查断言前操作是否正常完成')
ERROR_MSG_0022 = (322, '页面无此元素，请检查传入的元素是否正确')
ERROR_MSG_0023 = (323, 'iframe中未找到此元素，请检查元素表达式是否是正确的')
ERROR_MSG_0024 = (324, '上传文件的元素必须要是input标签中的')
ERROR_MSG_0025 = (325, '无法元素对应的下标，请检查通过下标获取元素的数是否正确，当前元素个数：{}')
ERROR_MSG_0026 = (326, '文件不存在')
ERROR_MSG_0027 = (327,
                  '您元素的操作内容中没有任何的数据，请检查：1.页面步骤详情中字段->元素操作值是否是空，是空可能是你删除了，也可能是执行器的操作选项没有同步需要点击执行器的同步发送缓存数据；2.元素表达式错误导致查询不到元素；')
ERROR_MSG_0028 = (328, '一个元素内只能包含一个需要被替换的内容')
ERROR_MSG_0029 = (329, '页面无元素：{} 表达式：{}')
ERROR_MSG_0030 = (330, '断言时没有找到元素')
ERROR_MSG_0031 = (331, '元素未找到准备进行断言获取元素文本类容异常')
ERROR_MSG_0032 = (332, '元素：{} 的元素表达式定位的标签无法对应操作的类型，或其他操作异常')
ERROR_MSG_0033 = (333, 'mysql中无：{}库')
ERROR_MSG_0034 = (334, 'sql存在语法错误，sql：{}')
ERROR_MSG_0035 = (335, 'sql语句是空，或查询字段或条件字段不存在')
ERROR_MSG_0036 = (336, '公共参数sql在数据库中查询不到结果，sql：{}')
ERROR_MSG_0037 = (337, '用例前置sql在数据库中查询不到结果，sql：{}')
ERROR_MSG_0038 = (338, '公共参数的sql_key不是列表')
ERROR_MSG_0039 = (339, '用例前置的sql_key不是列表')
ERROR_MSG_0040 = (
    340, '设备启动超时！请检查设备是否开启了：文件传输模式、开发者模式、usb调试，开启后请重新连接电脑，设备号：{}')
ERROR_MSG_0041 = (341, '查找页面元素时直接失败，元素：{}，报错信息：{}')
ERROR_MSG_0042 = (342, '实例化对象错误，设备信息是空无法实例化，请联系管理员检查')
ERROR_MSG_0043 = (343, '元素可能不存在，元素：{}，报错信息：{}')
ERROR_MSG_0044 = (344, '元素可能无法消失，元素：{}，报错信息：{}')
ERROR_MSG_0045 = (345, '设备启动超时！请检查设备是否已成功连接电脑，设备号：{}')
ERROR_MSG_0046 = (346, '您要测试的设备中，没有您输入的包名，请检查包名是否正确，或者设备是否安装了这个软件包')
ERROR_MSG_0047 = (347, '方法不存在，请检查输入的方法名称是否正确')
ERROR_MSG_0048 = (348, '选项发生错误，请在执行器点击发送缓存数据，然后重新修改步骤的操作！或联系管理员处理！')
ERROR_MSG_0049 = (349, 'url格式错误，请检查url或者是选择的测试环境不正确')
ERROR_MSG_0050 = (350, 'xpath定位未找到元素')
ERROR_MSG_0051 = (351, '选择的是元素操作，但是你没有选择一个<元素操作>选项，所以不知道你要操作什么')
ERROR_MSG_0052 = (352, '元素：{} 进行断言时发生异常，请检查元素是否可以正常使用')
ERROR_MSG_0053 = (353, '步骤是断言类型，但是你没有输入断言内容，请查看文档中对元素断言的介绍')
ERROR_MSG_0054 = (354, '步骤是操作类型，但是你没有输入操作内容，请查看文档中对元素操作的介绍')
ERROR_MSG_0055 = (355, '您的电脑未安装指定浏览器！')
ERROR_MSG_0056 = (356, '输入的参数类型不正确，请检查操作输入框的类型是否正确')
ERROR_MSG_0057 = (
    357,
    '创建浏览器的时候失败,如果你开启了视频录制，则需要执行帮助文档中，windows部署的模块最后的安装playwright依赖。如果不是视频录制原因，则请发给管理员！')
ERROR_MSG_0058 = (358, '电脑可能无网络，或者设置了代理，浏览器无法进行请求！')
ERROR_MSG_0059 = (359, '输入内容的类型不正确')
ERROR_MSG_0060 = (360, '请把元素等信息都发给管理员来解决这个报错，{}')
ERROR_MSG_0061 = (361, '前置或者后置的自定义函数没有按照文档规范填写，或者语法不正确，请参照文档后重新填写')
ERROR_MSG_0062 = (362, '请执行：playwright install 命令，安装符合要求的浏览器再运行命令行模式的执行器')



# -*- coding: utf-8 -*-
# @Project: 芒果测试平台
# @Description: # @Time   : 2023-04-26 22:22
# @Author : 毛鹏
import asyncio
import os

import time
from playwright.async_api import Locator, Error

from mangotools.decorator import async_method_callback
from mangotools.models import MethodModel
from ..._base_data import BaseData
from ....exceptions import MangoAutomationError
from ....exceptions._error_msg import ERROR_MSG_0024, ERROR_MSG_0056
from ....tools import Meta


class AsyncWebElement(metaclass=Meta):
    """元素操作"""

    def __init__(self, base_data: BaseData):
        self.base_data = base_data

    @async_method_callback('web', '元素操作', 0, [MethodModel(f='locating')])
    async def w_click(self, locating: Locator):
        """元素单击"""
        await locating.click()

    @async_method_callback('web', '元素操作', 1, [MethodModel(f='locating')])
    async def w_dblclick(self, locating: Locator):
        """元素双击"""
        await locating.dblclick()

    @async_method_callback('web', '元素操作', 2, [MethodModel(f='locating')])
    async def w_force_click(self, locating: Locator):
        """强制单击"""
        await locating.evaluate('element => element.click()')

    @async_method_callback('web', '元素操作', 3, [
        MethodModel(f='locating'),
        MethodModel(f='input_value', p='请输入输入内容', d=True)])
    async def w_input(self, locating: Locator, input_value: str):
        """元素输入"""
        await locating.fill(str(input_value))

    @async_method_callback('web', '元素操作', 4, [MethodModel(f='locating')])
    async def w_hover(self, locating: Locator):
        """鼠标悬停"""
        await locating.hover()
        await asyncio.sleep(1)

    @async_method_callback('web', '元素操作', 5, [
        MethodModel(f='locating'),
        MethodModel(f='set_cache_key', p='请输入获取元素文本后存储的key', d=True)])
    async def w_get_text(self, locating: Locator, set_cache_key=None):
        """获取元素文本"""
        value = await locating.inner_text()
        if set_cache_key:
            self.base_data.test_data.set_cache(key=set_cache_key, value=value)

        return value

    @async_method_callback('web', '元素操作', 6, [
        MethodModel(f='locating'),
        MethodModel(f='file_path', p='请输入文件路径，参照帮助文档', d=True)])
    async def w_upload_files(self, locating: Locator, file_path: str | list):
        """拖拽文件上传"""
        try:
            if isinstance(file_path, str):
                await locating.set_input_files(file_path)
            else:
                for file in file_path:
                    await locating.set_input_files(file)
        except Error:
            raise MangoAutomationError(*ERROR_MSG_0024)

    @async_method_callback('web', '元素操作', 7, [
        MethodModel(f='locating'),
        MethodModel(f='file_path', p='请输入文件路径，参照帮助文档', d=True)])
    async def w_click_upload_files(self, locating: Locator, file_path: str | list):
        """点击并选择文件上传"""
        async with self.base_data.page.expect_file_chooser() as fc_info:
            await locating.click()
        file_chooser = await fc_info.value
        await file_chooser.set_files(file_path)

    @async_method_callback('web', '元素操作', 8, [
        MethodModel(f='locating'),
        MethodModel(f='file_key', p='请输入文件存储路径的key，后续通过key获取文件保存的绝对路径', d=True)])
    async def w_download(self, locating: Locator, file_key: str):
        """下载文件"""
        async with self.base_data.page.expect_download() as download_info:
            await locating.click()
        download = await download_info.value
        file_name = download.suggested_filename
        save_path = os.path.join(self.base_data.download_path, file_name)
        await download.save_as(save_path)
        self.base_data.test_data.set_cache(file_key, file_name)

    @async_method_callback('web', '元素操作', 9, [
        MethodModel(f='locating1'),
        MethodModel(f='locating2')])
    async def w_drag_to(self, locating1: Locator, locating2: Locator):
        """拖动A元素到达B"""
        await locating1.drag_to(locating2)

    @async_method_callback('web', '元素操作', 10, [
        MethodModel(f='locating'),
        MethodModel(f='n', p='请输入循环点击的时间', d=True)])
    async def w_time_click(self, locating: Locator, n: int):
        """循环点击N秒"""
        try:
            n = int(n)
        except ValueError:
            raise MangoAutomationError(*ERROR_MSG_0056)
        s = time.time()
        while True:
            await locating.click()
            if time.time() - s > n:
                return

    @async_method_callback('web', '元素操作', 11, [
        MethodModel(f='locating'),
        MethodModel(f='n', p='请输入向上像素', d=True)])
    async def w_drag_up_pixel(self, locating: Locator, n: int):
        """往上拖动N个像素"""
        try:
            n = int(n)
        except ValueError:
            raise MangoAutomationError(*ERROR_MSG_0056)

        box = await locating.bounding_box()

        if box:
            await self.base_data.page.mouse.move(box['x'] + box['width'] / 2, box['y'] + box['height'] / 2)
            await self.base_data.page.mouse.down()
            await self.base_data.page.mouse.move(box['x'] + box['width'] / 2, box['y'] + box['height'] / 2 - n)
            await self.base_data.page.mouse.up()

    @async_method_callback('web', '元素操作', 12, [
        MethodModel(f='locating'),
        MethodModel(f='n', p='请输入向下像素', d=True)])
    async def w_drag_down_pixel(self, locating: Locator, n: int):
        """往下拖动N个像素"""
        try:
            n = int(n)
        except ValueError:
            raise MangoAutomationError(*ERROR_MSG_0056)

        box = await locating.bounding_box()

        if box:
            await self.base_data.page.mouse.move(box['x'] + box['width'] / 2, box['y'] + box['height'] / 2)
            await self.base_data.page.mouse.down()
            await self.base_data.page.mouse.move(box['x'] + box['width'] / 2, box['y'] + box['height'] / 2 + n)
            await self.base_data.page.mouse.up()

    @async_method_callback('web', '元素操作', 13, [
        MethodModel(f='locating'),
        MethodModel(f='n', p='请输入向左像素', d=True)])
    async def w_drag_left_pixel(self, locating: Locator, n: int):
        """往左拖动N个像素"""
        try:
            n = int(n)
        except ValueError:
            raise MangoAutomationError(*ERROR_MSG_0056)

        box = await locating.bounding_box()

        if box:
            await self.base_data.page.mouse.move(box['x'] + box['width'] / 2, box['y'] + box['height'] / 2)
            await self.base_data.page.mouse.down()
            await self.base_data.page.mouse.move(box['x'] + box['width'] / 2 - n, box['y'] + box['height'] / 2)
            await self.base_data.page.mouse.up()

    @async_method_callback('web', '元素操作', 14, [
        MethodModel(f='locating'),
        MethodModel(f='n', p='请输入向右像素', d=True)])
    async def w_drag_right_pixel(self, locating: Locator, n: int):
        """往右拖动N个像素"""
        try:
            n = int(n)
        except ValueError:
            raise MangoAutomationError(*ERROR_MSG_0056)
        box = await locating.bounding_box()

        if box:
            await self.base_data.page.mouse.move(box['x'] + box['width'] / 2, box['y'] + box['height'] / 2)
            await self.base_data.page.mouse.down()
            await self.base_data.page.mouse.move(box['x'] + box['width'] / 2 + n, box['y'] + box['height'] / 2)
            await self.base_data.page.mouse.up()

    @async_method_callback('web', '元素操作', 15, [
        MethodModel(f='locating'),
        MethodModel(f='path', p='请输入截图保存路径', d=True)])
    async def w_ele_screenshot(self, locating: Locator, path: str):
        """元素截图"""
        await locating.screenshot(path=path)



# -*- coding: utf-8 -*-
# @Project: 芒果测试平台
# @Description: # @Time   : 2023-04-29 12:11
# @Author : 毛鹏
from mangotools.decorator import sync_method_callback
from mangotools.models import MethodModel
from ....tools import Meta
from ....uidrive._base_data import BaseData


class SyncWebDeviceInput(metaclass=Meta):
    """输入设备"""

    def __init__(self, base_data: BaseData):
        self.base_data = base_data

    @sync_method_callback('web', '输入设备', 0, [
        MethodModel(f='keyboard', p='请输入键盘名称，首字母大写', d=True)])
    def w_keys(self, keyboard: str):
        """模拟按下指定的键"""
        self.base_data.page.keyboard.press(str(keyboard))

    @sync_method_callback('web', '输入设备', 1, [
        MethodModel(f='y', p='请输入向上滚动像素', d=True)])
    def w_wheel(self, y):
        """鼠标上下滚动像素，负数代表向上"""
        self.base_data.page.mouse.wheel(0, y)

    @sync_method_callback('web', '输入设备', 2, [
        MethodModel(f='x', p='请输入点击的x轴', d=True), MethodModel(f='y', p='请输入点击的y轴', d=True)])
    def w_mouse_click(self, x: float, y: float):
        """鼠标点击坐标"""
        self.base_data.page.mouse.click(x, y)

    @sync_method_callback('web', '输入设备', 3)
    def w_mouse_center(self):
        """鼠标移动到中间"""

        viewport_size = self.base_data.page.evaluate('''() => {
            return {
                width: window.innerWidth,
                height: window.innerHeight
            }
        }''')
        center_x = viewport_size['width'] / 2
        center_y = viewport_size['height'] / 2
        self.base_data.page.mouse.move(center_x, center_y)

    @sync_method_callback('web', '输入设备', 4)
    def w_mouse_center(self):
        """鼠标移动到中间并点击"""

        viewport_size = self.base_data.page.evaluate('''() => {
            return {
                width: window.innerWidth,
                height: window.innerHeight
            }
        }''')
        center_x = viewport_size['width'] / 2
        center_y = viewport_size['height'] / 2
        self.base_data.page.mouse.click(center_x, center_y)

    @sync_method_callback('web', '输入设备', 5, [MethodModel(f='text', p='请输入键盘输入的内容', d=True)])
    def w_keyboard_type_text(self, text: str):
        """模拟人工输入文字"""
        self.base_data.page.keyboard.type(str(text))

    @sync_method_callback('web', '输入设备', 6, [
        MethodModel(f='text', p='请输入键盘输入的内容', d=True)])
    def w_keyboard_insert_text(self, text: str):
        """直接输入文字"""
        self.base_data.page.keyboard.insert_text(str(text))

    @sync_method_callback('web', '输入设备', 7, [
        MethodModel(f='count', p='请输入要删除字符串的个数', d=True)])
    def w_keyboard_delete_text(self, count: int):
        """删除光标左侧的字符"""
        for _ in range(0, int(count) + 1):
            self.base_data.page.keyboard.press("Backspace")



from setuptools import setup, find_packages

__version__ = '1.0.0'

with open("README.md", "r", encoding='utf-8') as fh:
    long_description = fh.read()

setup(
    name='mangoautomation',
    version=__version__,
    description='测试工具',
    long_description=long_description,
    package_data={ },
    author='毛鹏',
    author_email='729164035@qq.com',
    url='https://gitee.com/mao-peng/testkit',
    packages=find_packages(),
    install_requires=[
        'setuptools==68.2.0',
        'pydantic==2.9.2',
        'playwright==1.43.0',
        'uiautomation==2.0.20',
        'uiautomator2==3.2.5',
        'mangotools==1.0.1',
        'adbutils~=2.8.9',

    ],
    classifiers=[
        "Programming Language :: Python :: 3.10",
    ]
)

"""


python -m pip install --upgrade setuptools wheel
python -m pip install --upgrade twine

python setup.py check
python setup.py sdist bdist_wheel
twine upload --repository-url https://upload.pypi.org/legacy/ dist/*

"""
